(use-translation _ "alterator-users")

(on-prev (goto "/users"))
(on-next (goto "/users"))

(define (on-add-error reason)
 (cond ((string=? reason "passwords mismatch")
		(run-dialog '("/users/mismatch")))
       ((string=? reason "wrong name format")
		(run-dialog '("/users/error")))
       (else (run-dialog '("/error")))))

(define (on-load-error reason)
  '("<b>error loading userlist</b>"))

(define (get-userlist)
  (woo-cache-ignore)
  (woo-read-names "/users" (woo-list "/users")))

(define *users* (woo-catch get-userlist on-load-error))

(if (null? *users*) ((global 'disable-next)))

(define (made-title-text userlist)
  (if (null? userlist) 
    (_ "No system users yet")
    (string-append (_ "Already added: ") (string-join userlist ", "))))

;information title
(define (make-title)
  (let ((userlist *users*))
    (id 'users-info
	(textbox
	  (made-title-text *users*)
	  readonly "yes"
	  height 5
	  (relative-width 90 page)))))

;add user only if username and two passwd non empty
(define (can-enable-add)
  (add-button enabled (and (not-empty-string? (username text))
			   (not-empty-string? (passwd1 text))
			   (not-empty-string? (passwd2 text)))))

;add new user to system
(define (add-user)
  (woo-catch
    (lambda()
      (woo-new
	(string-append "/users/" (username text))
	'type "local"
	'gecos (gecos text)
	'passwd1 (passwd1 text)
	'passwd2 (passwd2 text)
	'allow_su (if (allow_su checked) "yes" "no"))
      (gecos text "")
      (username text "")
      (passwd1 text "")
      (passwd2 text "")
      (allow_su checked #f)
      (let ((lst (get-userlist)))
	(next-button enabled  (if (null? lst) "no" "yes"))
	(users-info text (made-title-text lst))
	(set! *users* lst)))
    on-add-error))
 
(define (get-default name)
  (let ((saved (global (string->symbol (string-append "saved_" (symbol->string name))))))
    (if saved saved "")))

(vbox sizepolicy 'fixed
      (make-title)
      (hbox spacing 1 margin 1
	    (vbox
	      (label (_ "Login name:") height 3)
	      (label (_ "User Password:") height 3)
	      (label (_ "User Password (retype):") height 3)
	      (label (_ "Comment:") height 3))
	    (vbox
	      (id 'username (edit (get-default 'name) height 3 (on-change (can-enable-add))))
	      (id 'passwd1 (edit "" echo stars height 3 (on-change (can-enable-add))))
	      (id 'passwd2 (edit "" echo stars height 3 (on-change (can-enable-add))))
	      (id 'gecos (edit (get-default 'gecos) height 3 (on-change (can-enable-add))))))
      (id 'allow_su (checkbox (_ "Allow user to obtain administrator's privileges (su)")
				      checked (null? *users*)))
      (id 'add-button (button (_ "Accept user")
			    enabled #f
			    (relative-width 50 page)
			    (on-click
			      (if (member (username text) *users*)
				(run-dialog `(/warning message ,(_ "User already exists")))
				(add-user))))))

