(use-translation _ "alterator-users")

(include-as-is "/dialog/functions")

(define *last-selected-user* #f)
(define *last-selected-tab* #f)
(define (selected-user) (userlist text))
(define (selected-user-cmd)
  (string-append "/users/" (selected-user)))

(define (get-users)
  (woo-read-names "/users"
		  (woo-list "/users")))

(define (get-user-info user)
  (woo-first-command
    (woo-read (string-append "/users/" user))))

(define (on-load-error reason) '())

(define *userlist* (woo-catch
		     get-users
		     on-load-error))

(define *on-leave* #f)

(on-acc-leave
  (if (next-button enabled) 'need-confirm))

(on-apply
  ((users-tabbox current) 'call on-apply))

((global 'disable-next))

(define (on-update-error reason)
  (username text (string-append "Error:" reason)))

(define (make-userlist)
  (id 'userlist
      (listbox (items *userlist*)
	       sizepolicy '(fixed . expanding)
	       width 15
	       (on-select
		 (if (and (next-button enabled)
			  (run-dialog '("/users/confirm")))
		   (apply-user-info))
		 (set! *last-selected-user* (selected-user-cmd))
		 (woo-catch
		   (lambda()
		     (update-user-info (selected-user)))
		   on-update-error)))))

(define (update-user-info name)
  (let ((user-info (get-user-info name)))
    (username text (userlist text))
    (gecos text (woo-get-option user-info 'gecos))
    (uid text (woo-get-option user-info 'uid))
    (gid text (woo-get-option user-info 'gid))
    (home text (woo-get-option user-info 'home))
    (check-buttons (> (userlist count) 0))
    (next-button enabled #f))) ;remove dirty flag after such change

(define (apply-user-info)
  (woo-catch
    (lambda()
      (woo-write *last-selected-user*
		 'type "local"
		 'gecos (gecos text)
		 'uid (uid text)
		 'gid (gid text)
		 'home (home text))
      (next-button enabled #f)
      #t)
    (lambda(reason)
      (run-dialog '("/users/error"))
      #f)))

(define (labeled-label name txt val)
  (hbox (label txt (relative-width 30 page))
	(id name (label val (relative-width 40 page)))))

(define (check-buttons is-pair?)
  (gecos enabled is-pair?)
  (uid enabled is-pair?)
  (gid enabled is-pair?)
  (home enabled is-pair?)
  (del-button enabled is-pair?)
  (chp-button enabled is-pair?))

(define (update-userlist)
  (let ((new-userlist (get-users)))
    (userlist (items new-userlist))
    (userlist current 0)
    (and (pair? new-userlist) (update-user-info (car new-userlist)))
    (check-buttons (pair? new-userlist))))

(define (delete-user)
  (if (run-dialog `("/yesno"
		    message
		    ,(string-append (_ "Do you really want to delete user")
				    (selected-user)
				    " ?")))
    (woo-catch
      (lambda()
	(woo-delete (selected-user-cmd) 'type "local")
	(update-userlist))
      on-update-error)))

(define (make-users-tab)
(vbox spacing 1 margin 1
      (hbox spacing 1 margin 1
	    (make-userlist)
	    (vbox
	      (hbox sizepolicy '(expanding . maximum)
		(vbox
		  (map
		    (lambda (x)
		      (label x height 3 font bold-font))
		    (list (_ "Login name:")  (_ "User ID:") (_ "Group ID:") (_ "Home:") (_ "Comment:") )))
		(vbox
		  (id 'username (label "" height 3))
		  (map
		    (lambda (x)
		      (id x (edit "" sizepolicy '(expanding . maximum)
				  enabled #f
				  height 3
				  (on-change (next-button enabled #t)))))
		    '(uid gid home gecos))))))
      (hbox sizepolicy '(expanding . maximum)
	(button (_ "New User")
		sizepolicy 'expanding
		(on-click (goto "/users/new")))
	(id 'del-button
	    (button (_ "Delete") 
		    sizepolicy 'expanding
		    enabled #f
		    (on-click (delete-user))))
	(id 'chp-button
	    (button (_ "Change password...")
		    sizepolicy 'expanding
		    enabled #f
		    (on-click (goto "/users/chpasswd"
				    'username (selected-user))))))))

(id 'users-tabbox
    (tabbox
      (on-change
	(if (and (not *on-leave*)
		 (next-button enabled)
		 (run-dialog '("/users/confirm")))
	  (*last-selected-tab* 'call on-apply))
	(set! *last-selected-tab* (users-tabbox current)))
      (tab 'users-tab (_ "System users")
           (on-destroy
	     (set! *on-leave* #t))
	   (on-apply
	     (apply-user-info) #t)
	   (make-users-tab))
      (tab 'chrootpasswd-tab (_ "Root password")
           (on-destroy
	     (set! *on-leave* #t))
	   (load-url "/chrootpasswd"))
      (current users-tab)))

(set! *last-selected-tab* users-tab)

